@@ -10,6 +10,7 @@ from warnings import warn
 
 from dacite import from_dict
 
+from onapsdk.aai.business.owning_entity import OwningEntity
 from onapsdk.exceptions import (
     APIError, InvalidResponse, ParameterError, ResourceNotFound, StatusError
 )
@@ -19,10 +20,26 @@ from onapsdk.sdc.service import Network, Service as SdcService, Vnf, VfModule
 from onapsdk.utils.jinja import jinja_env
 from onapsdk.utils.headers_creator import headers_so_creator
 from onapsdk.vid import LineOfBusiness, Platform
+from onapsdk.configuration import settings
 
 from .so_element import OrchestrationRequest
 
 
+@dataclass
+class Operation:
+    """Operation class with data about method and suffix for VnfOperation."""
+
+    request_method: str
+    request_suffix: str
+
+
+class VnfOperation(Operation):  # pylint: disable=too-few-public-methods
+    """Class to store possible operations' data for vnfs (request method and suffix)."""
+
+    UPDATE = Operation("PUT", "")
+    HEALTHCHECK = Operation("POST", "/healthcheck")
+
+
 @dataclass
 class SoServiceVfModule:
     """Class to store a VfModule instance parameters."""
@@ -42,6 +59,18 @@ class SoServiceXnf:
     parameters: Dict[str, Any] = field(default_factory=dict)
     processing_priority: Optional[int] = None
 
+    @classmethod
+    def load(cls, data: Dict[str, Any]) -> "SoServiceVnf":
+        """Create a vnf instance description object from the dict.
+
+        Useful if you keep your instance data in file.
+
+        Returns:
+            SoServiceVnf: SoServiceVnf object created from the dictionary
+
+        """
+        return from_dict(data_class=cls, data=data)
+
 
 @dataclass
 class SoServiceVnf(SoServiceXnf):
@@ -270,6 +299,7 @@ class VfModuleInstantiation(Instantiation):  # pytest: disable=too-many-ancestor
             vf_module=vf_module
         )
 
+
 class NodeTemplateInstantiation(Instantiation, ABC):  # pytest: disable=too-many-ancestors
     """Base class for service's node_template object instantiation."""
 
@@ -457,6 +487,168 @@ class VnfInstantiation(NodeTemplateInstantiation):  # pylint: disable=too-many-a
             vnf=vnf_object
         )
 
+    @classmethod
+    def instantiate_macro(cls,  # pylint: disable=too-many-arguments, too-many-locals
+                          aai_service_instance: "ServiceInstance",
+                          vnf_object: "Vnf",
+                          line_of_business: str,
+                          platform: str,
+                          cloud_region: "CloudRegion",
+                          tenant: "Tenant",
+                          sdc_service: "SdcService",
+                          vnf_instance_name: str = None,
+                          vnf_parameters: Iterable["InstantiationParameter"] = None,
+                          so_vnf: "SoServiceVnf" = None
+                          ) -> "VnfInstantiation":
+        """Instantiate Vnf using macro method.
+
+        Args:
+            aai_service_instance (ServiceInstance): Service instance associated with request
+            vnf_object (Vnf): Vnf to instantiate
+            line_of_business (LineOfBusiness): LineOfBusiness to use in instantiation request
+            platform (Platform): Platform to use in instantiation request
+            cloud_region (CloudRegion): Cloud region to use in instantiation request.
+            tenant (Tenant): Tenant to use in instantiation request.
+            vnf_instance_name (str, optional): Vnf instance name. Defaults to None.
+            vnf_parameters (Iterable[InstantiationParameter], optional): Instantiation parameters
+                that are sent in the request. Defaults to None
+            so_vnf (SoServiceVnf): object with vnf instance parameters
+
+        Returns:
+            VnfInstantiation: VnfInstantiation object
+
+        """
+        owning_entity_id = None
+        project = settings.PROJECT
+
+        for relationship in aai_service_instance.relationships:
+            if relationship.related_to == "owning-entity":
+                owning_entity_id = relationship.relationship_data.pop().get("relationship-value")
+            if relationship.related_to == "project":
+                project = relationship.relationship_data.pop().get("relationship-value")
+
+        owning_entity = OwningEntity.get_by_owning_entity_id(
+            owning_entity_id=owning_entity_id)
+
+        if so_vnf:
+            template_file = "instantiate_vnf_macro_so_vnf.json.j2"
+            if so_vnf.instance_name:
+                vnf_instance_name = so_vnf.instance_name
+        else:
+            template_file = "instantiate_vnf_macro.json.j2"
+            if vnf_instance_name is None:
+                vnf_instance_name = \
+                    f"Python_ONAP_SDK_vnf_instance_{str(uuid4())}"
+
+        response: dict = cls.send_message_json(
+            "POST",
+            (f"Instantiate {sdc_service.name} "
+             f"service vnf {vnf_object.name}"),
+            (f"{cls.base_url}/onap/so/infra/serviceInstantiation/{cls.api_version}/"
+             f"serviceInstances/{aai_service_instance.instance_id}/vnfs"),
+            data=jinja_env().get_template(template_file).render(
+                instance_name=vnf_instance_name,
+                vnf=vnf_object,
+                service=sdc_service,
+                cloud_region=cloud_region or \
+                             next(aai_service_instance.service_subscription.cloud_regions),
+                tenant=tenant or next(aai_service_instance.service_subscription.tenants),
+                project=project,
+                owning_entity=owning_entity,
+                line_of_business=line_of_business,
+                platform=platform,
+                service_instance=aai_service_instance,
+                vnf_parameters=vnf_parameters or [],
+                so_vnf=so_vnf
+            ),
+            headers=headers_so_creator(OnapService.headers)
+        )
+
+        return VnfInstantiation(
+            name=vnf_instance_name,
+            request_id=response["requestReferences"]["requestId"],
+            instance_id=response["requestReferences"]["instanceId"],
+            line_of_business=line_of_business,
+            platform=platform,
+            vnf=vnf_object
+        )
+
+    @classmethod
+    def so_action(cls,  # pylint: disable=too-many-arguments, too-many-locals
+                  vnf_instance: "VnfInstance",
+                  operation_type: VnfOperation,
+                  aai_service_instance: "ServiceInstance",
+                  line_of_business: str,
+                  platform: str,
+                  sdc_service: "SdcService",
+                  so_service: "SoService" = None
+                  ) -> "VnfInstantiation":
+        """Execute SO action (update or healthcheck) for selected vnf with SO macro request.
+
+        Args:
+            vnf_instance (VnfInstance): vnf instance object
+            operation_type (VnfOperation): name of the operation to trigger
+            aai_service_instance (AaiService): Service Instance object from aai
+            line_of_business (LineOfBusiness): LineOfBusiness name to use
+                in instantiation request
+            platform (Platform): Platform name to use in instantiation request
+            sdc_service (SdcService): Service model information
+            so_service (SoService, optional): SO values to use in SO request
+
+        Raises:
+            StatusError: if the provided operation is not supported
+
+        Returns:
+            VnfInstantiation: VnfInstantiation object
+
+        """
+        if operation_type not in (VnfOperation.HEALTHCHECK, VnfOperation.UPDATE):
+            raise StatusError("Operation not supported!")
+
+        owning_entity_id = None
+        project = settings.PROJECT
+
+        for relationship in aai_service_instance.relationships:
+            if relationship.related_to == "owning-entity":
+                owning_entity_id = relationship.relationship_data.pop().get("relationship-value")
+            if relationship.related_to == "project":
+                project = relationship.relationship_data.pop().get("relationship-value")
+
+        owning_entity = OwningEntity.get_by_owning_entity_id(
+            owning_entity_id=owning_entity_id)
+
+        response: dict = cls.send_message_json(
+            operation_type.request_method,
+            (f"So Action {sdc_service.name} "
+             f" vnf instance {vnf_instance.vnf_id}"),
+            (f"{cls.base_url}/onap/so/infra/serviceInstantiation/{cls.api_version}/"
+             f"serviceInstances/{aai_service_instance.instance_id}/vnfs/{vnf_instance.vnf_id}"
+             f"{operation_type.request_suffix}"),
+            data=jinja_env().get_template("instantiate_multi_vnf_service_macro.json.j2").render(
+                sdc_service=sdc_service,
+                cloud_region=next(aai_service_instance.service_subscription.cloud_regions),
+                tenant=next(aai_service_instance.service_subscription.tenants),
+                customer=aai_service_instance.service_subscription.customer,
+                project=project,
+                owning_entity=owning_entity,
+                line_of_business=line_of_business,
+                platform=platform,
+                service_instance_name=aai_service_instance.instance_name,
+                so_service=so_service
+            ),
+            headers=headers_so_creator(OnapService.headers)
+        )
+
+        return VnfInstantiation(
+            name=vnf_instance.vnf_name,
+            request_id=response["requestReferences"]["requestId"],
+            instance_id=response["requestReferences"]["instanceId"],
+            line_of_business=line_of_business,
+            platform=platform,
+            vnf=vnf_instance
+        )
+
+
 class ServiceInstantiation(Instantiation):  # pylint: disable=too-many-ancestors
     """Service instantiation class."""
 
@@ -692,7 +884,7 @@ class ServiceInstantiation(Instantiation):  # pylint: disable=too-many-ancestors
 
     @property
     def aai_service_instance(self) -> "ServiceInstance":
-        """Service instane associated with service instantiation request.
+        """Service instance associated with service instantiation request.
 
         Raises:
             StatusError: if a service is not instantiated -
